@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2003 Imendio HB
  * Copyright (C) 2002 Richard Hult <richard@imendio.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,9 +27,9 @@
 
 typedef struct _GossipGroupChat GossipGroupChat;
 
-GossipGroupChat *gossip_group_chat_new        (GossipJID    *jid,
-					       const gchar  *nick);
-void             gossip_group_chat_set_status (GossipStatus  status);
+GossipGroupChat *gossip_group_chat_new      (GossipJID   *jid,
+					     const gchar *nick);
+void             gossip_group_chat_set_show (GossipShow   show);
 
 
 #endif /* __GOSSIP_GROUP_CHAT_H__ */
