@@ -164,7 +164,7 @@ join_dialog_response_cb (GtkWidget        *dialog,
 		lm_connection_send (join->connection, m, NULL);
 		lm_message_unref (m);
 
-		gossip_app_join_group_chat (join->app, room, server, nick);
+		gossip_app_join_group_chat (room, server, nick);
 		g_free (nick);
 		break;
 
@@ -295,6 +295,7 @@ gossip_join_dialog_new (GossipApp *app)
 		
 	join_dialog_update_option_menu (join);
 
+	gtk_window_set_transient_for (GTK_WINDOW (join->dialog), GTK_WINDOW (gossip_app_get_window ()));
 	gtk_widget_show (join->dialog);
 
 	return join;
