@@ -26,48 +26,44 @@
 #include "gossip-app.h"
 #include "gossip-sound.h"
 
-/* Use gconf pref for sound preferences later. */
-
 extern GConfClient *gconf_client;
 
 void
 gossip_sound_play (GossipSound sound)
 {
-	gboolean      enabled, silent;
-	GossipStatus  status;
-	const gchar  *file;
-	gchar        *str;
+	gboolean     enabled, silent_busy, silent_away;
+	GossipShow   show;
+	const gchar *file;
+	gchar       *str;
+
+	if (!gossip_app_is_connected ()) {
+		return;
+	}
 	
 	enabled = gconf_client_get_bool (gconf_client,
 					 GCONF_PATH "/sound/play_sounds",
 					 NULL);
-	silent = gconf_client_get_bool (gconf_client,
-					GCONF_PATH "/sound/silent_away",
-					NULL);
+	silent_busy = gconf_client_get_bool (gconf_client,
+					     GCONF_PATH "/sound/silent_busy",
+					     NULL);
+	silent_away = gconf_client_get_bool (gconf_client,
+					     GCONF_PATH "/sound/silent_away",
+					     NULL);
 
 	if (!enabled) {
 		return;
 	}
-
-	if (silent) {
-		status = gossip_app_get_status ();
-		switch (status) {
-		case GOSSIP_STATUS_AVAILABLE:
-		case GOSSIP_STATUS_FREE:
-			break;
-			
-		case GOSSIP_STATUS_BUSY:
-		case GOSSIP_STATUS_AWAY:
-		case GOSSIP_STATUS_EXT_AWAY:
-		case GOSSIP_STATUS_OFFLINE:
-			return;
-
-		default:
-			g_assert_not_reached ();
-			break;
-		}
-	}
 	
+	show = gossip_app_get_show ();
+	
+	if (silent_busy && show == GOSSIP_SHOW_BUSY) {
+		return;
+	}
+
+	if (silent_away && (show == GOSSIP_SHOW_AWAY || show == GOSSIP_SHOW_EXT_AWAY)) {
+		return;
+	}
+
 	switch (sound) {
 	case GOSSIP_SOUND_CHAT:
 		file = "chat1.wav";
